@@ -102,7 +102,7 @@ use constant DEBUG => 0;
 
 use vars qw(@ISA $VERSION);
 @ISA = qw(Bio::Root::Root Bio::SeqI Bio::Das::SegmentI Bio::DB::Das::Chado);
-$VERSION = 0.26;
+$VERSION = 0.30;
 
 #use overload '""' => 'asString';
 
@@ -214,6 +214,8 @@ sub new {
 
     if ( ref $ref eq 'ARRAY' ) {    #more than one result returned
 
+        warn "\n\n@$ref\n\n";
+
         my @segments;
 
         foreach my $feature_id (@$ref) {
@@ -330,6 +332,7 @@ sub new {
 
             warn "base_start:$base_start, stop:$stop, length:$length" if DEBUG;
 
+            $self->feature_id($landmark_feature_id);
             $self->start($base_start);
             $self->end($stop);
             $self->{'length'} = $length;
@@ -423,6 +426,32 @@ sub feature_id {
   my $self = shift;
 
   return $self->{'feature_id'} = shift if @_;
+  return $self->{'feature_id'} if $self->{'feature_id'};
+
+  my $dbh = $self->factory->dbh;
+
+  warn $self->name;
+  warn $self->type;
+  $self->factory->name2term($self->type);
+
+  my $name    = $self->name;
+  my $org_id  = $self->factory->organism_id;
+  my $type_id = $self->factory->name2term($self->type); 
+
+  my $query = "SELECT feature_id FROM feature WHERE (name = ? OR uniquename = ?)
+                  AND type_id = ? ";
+
+  my @args = ($name,$name,$type_id);
+  if ($org_id) {
+      $query .= " AND organism_id = ?";
+      push @args, $org_id;
+  }
+  my $sth = $dbh->prepare($query);
+  $sth->execute(@args);
+  return if $sth->rows > 1;
+  
+  my ($feature_id) = $sth->fetchrow_array;
+  $self->{'feature_id'} = $feature_id;
   return $self->{'feature_id'};
 }
 
@@ -488,6 +517,8 @@ sub _search_by_name {
   my $self = shift;
   my ($factory,$quoted_name,$db_id,$feature_id) = @_;
 
+  my $fulltext = $factory->fulltext;
+
   warn "_search_by_name args:@_" if DEBUG;
 
   my $obsolete_part = "";
@@ -509,6 +540,7 @@ sub _search_by_name {
 
    } 
    else {
+    #can't use FTS here as exact names are required
     $sth = $factory->dbh->prepare ("
              select name,feature_id,seqlen from feature
              where lower(name) = $quoted_name $obsolete_part ");
@@ -527,20 +559,41 @@ sub _search_by_name {
     warn "looking for a synonym to $quoted_name" if DEBUG;
     my $isth;
     if ($self->factory->use_all_feature_names()) {
+
+      my $optional_full_text;
+      if ($fulltext) {
+          $optional_full_text 
+           = "afn.searchable_name @@ plainto_tsquery($quoted_name) $where_part";
+      }
+      else {
+          $optional_full_text
+           = "lower(afn.name) = $quoted_name $where_part";
+      }
+
       $isth = $factory->dbh->prepare ("
         select afn.feature_id from all_feature_names afn, feature f
         where afn.feature_id = f.feature_id and
         f.is_obsolete = 'false' and
-        lower(afn.name) = $quoted_name $where_part
+        $optional_full_text
       ");
+
     }
     else {
+      my $full_text_options;
+      if ($fulltext) {
+          $full_text_options
+           = "s.searchable_synonym_sgml @@ plainto_tsquery($quoted_name) $where_part";
+      }
+      else {
+          $full_text_options
+           = "lower(s.synonym_sgml) = $quoted_name $where_part";
+      }
       $isth = $factory->dbh->prepare ("
         select fs.feature_id from feature_synonym fs, synonym s, feature f
         where fs.synonym_id = s.synonym_id and
         f.feature_id = fs.feature_id and
         f.is_obsolete = 'false' and 
-        lower(s.synonym_sgml) = $quoted_name $where_part
+        $full_text_options
       ");
     }
     $isth->execute or Bio::Root::Root->throw("query for name in synonym failed");
@@ -549,12 +602,20 @@ sub _search_by_name {
     if ($rows_returned == 0) { #look in dbxref for accession number match
       warn "looking in dbxref for $quoted_name" if DEBUG;
 
+      my $full_text_option;
+      if ($fulltext) {
+          $full_text_option = "d.searchable_accession @@ plainto_tsquery($quoted_name) $where_part";
+      }
+      else {
+          $full_text_option = "lower(d.accession) = $quoted_name $where_part";
+      }
+
       $isth = $factory->dbh->prepare ("
          select fd.feature_id from feature_dbxref fd, dbxref d, feature f
          where fd.dbxref_id = d.dbxref_id and
                f.feature_id = fd.feature_id and
                f.is_obsolete = 'false' and
-               lower(d.accession) = $quoted_name $where_part");
+               $full_text_option");
       $isth->execute or Bio::Root::Root->throw("query for accession failed");
       $rows_returned = $isth->rows;
 
@@ -614,20 +675,12 @@ sub _search_by_name {
 
 =head2 class
 
-  Title   : class
-  Usage   : $obj->class($newval)
-  Function: Returns the segment class (synonymous with type)
-  Returns : value of class (a scalar)
-  Args    : on set, new value (a scalar or undef, optional)
-
+Needed for backward compatability; always returns 'Sequence'.
 
 =cut
 
 sub class {
-  my $self = shift;
-
-  return $self->{'class'} = shift if @_;
-  return $self->{'class'};
+    return 'Sequence';
 }
 
 =head2 type
