@@ -30,7 +30,7 @@ use constant DEBUG => 0;
 use vars qw(@ISA $AUTOLOAD %CONSTANT_TAGS $VERSION);
 @ISA = qw(Bio::DB::Das::Chado::Segment Bio::SeqFeatureI
           Bio::Root::Root);
-$VERSION = 0.26;
+$VERSION = 0.30;
 
 %CONSTANT_TAGS = ();
 
@@ -626,15 +626,6 @@ methods described below, L<Bio::RangeI> for details.
 
 *abs_strand = \&strand;
 
-=head2 class()
-
-  Title   : class
-  Function: aliased to method()for backward compatibility
-
-=cut
-
-*class = \&type;
-
 =head2 db_id()
 
   Title   : db_id
